==> create-fly-1.21.11-pre1-6.0.8-1-sources.jar: com/zurrtum/create/content/redstone/link/RedstoneLinkBlock.java <==
package com.zurrtum.create.content.redstone.link;

import com.zurrtum.create.AllBlockEntityTypes;
import com.zurrtum.create.AllShapes;
import com.zurrtum.create.catnip.data.Iterate;
import com.zurrtum.create.foundation.block.IBE;
import com.zurrtum.create.foundation.block.NeighborUpdateListeningBlock;
import com.zurrtum.create.foundation.block.RedStoneConnectBlock;
import com.zurrtum.create.foundation.block.WrenchableDirectionalBlock;
import net.minecraft.class_10;
import net.minecraft.class_1269;
import net.minecraft.class_1657;
import net.minecraft.class_1750;
import net.minecraft.class_1838;
import net.minecraft.class_1922;
import net.minecraft.class_1937;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2591;
import net.minecraft.class_265;
import net.minecraft.class_2680;
import net.minecraft.class_2689;
import net.minecraft.class_2741;
import net.minecraft.class_2746;
import net.minecraft.class_3218;
import net.minecraft.class_3726;
import net.minecraft.class_3965;
import net.minecraft.class_4538;
import net.minecraft.class_5819;

public class RedstoneLinkBlock extends WrenchableDirectionalBlock implements IBE<RedstoneLinkBlockEntity>, RedStoneConnectBlock, NeighborUpdateListeningBlock {

    public static final class_2746 POWERED = class_2741.field_12484;
    public static final class_2746 RECEIVER = class_2746.method_11825("receiver");

    public RedstoneLinkBlock(class_2251 properties) {
        super(properties);
        method_9590(method_9564().method_11657(POWERED, false).method_11657(RECEIVER, false));
    }

    @Override
    public void neighborUpdate(class_2680 state, class_1937 level, class_2338 pos, class_2248 sourceBlock, class_2338 fromPos, boolean isMoving) {
        if (level.method_8608())
            return;
        if (fromPos.equals(pos.method_10093(state.method_11654(field_10927).method_10153()))) {
            if (!method_9558(state, level, pos)) {
                level.method_22352(pos, true);
                return;
            }
        }
        if (!level.method_8397().method_8677(pos, this))
            level.method_64310(pos, this, 1);
    }

    @Override
    public void method_9588(class_2680 state, class_3218 level, class_2338 pos, class_5819 r) {
        updateTransmittedSignal(state, level, pos);

        if (state.method_11654(RECEIVER))
            return;
        class_2350 attachedFace = state.method_11654(RedstoneLinkBlock.field_10927).method_10153();
        class_2338 attachedPos = pos.method_10093(attachedFace);
        level.method_8408(pos, level.method_8320(pos).method_26204());
        level.method_8408(attachedPos, level.method_8320(attachedPos).method_26204());
    }

    @Override
    public void method_9615(class_2680 state, class_1937 worldIn, class_2338 pos, class_2680 oldState, boolean isMoving) {
        if (state.method_26204() == oldState.method_26204() || isMoving)
            return;
        updateTransmittedSignal(state, worldIn, pos);
    }

    public void updateTransmittedSignal(class_2680 state, class_1937 level, class_2338 pos) {
        if (level.method_8608())
            return;
        if (state.method_11654(RECEIVER))
            return;

        int power = getPower(level, state, pos);
        int powerFromPanels = getBlockEntityOptional(level, pos).map(be -> {
            if (be.panelSupport == null)
                return 0;
            Boolean tri = be.panelSupport.shouldBePoweredTristate();
            if (tri == null)
                return -1;
            return tri ? 15 : 0;
        }).orElse(0);

        // Suppress update if an input panel exists but is not loaded
        if (powerFromPanels == -1)
            return;

        power = Math.max(power, powerFromPanels);

        boolean previouslyPowered = state.method_11654(POWERED);
        if (previouslyPowered != power > 0)
            level.method_8652(pos, state.method_28493(POWERED), class_2248.field_31028);

        int transmit = power;
        withBlockEntityDo(level, pos, be -> be.transmit(transmit));
    }

    private static int getPower(class_1937 level, class_2680 state, class_2338 pos) {
        int power = 0;
        for (class_2350 direction : Iterate.directions)
            power = Math.max(level.method_49808(pos.method_10093(direction), direction), power);
        for (class_2350 direction : Iterate.directions) {
            if (state.method_11654(field_10927).method_10153() != direction)
                power = Math.max(level.method_49808(pos.method_10093(direction), class_2350.field_11036), power);
        }
        return power;
    }

    @Override
    public boolean method_9506(class_2680 state) {
        return state.method_11654(POWERED) && state.method_11654(RECEIVER);
    }

    @Override
    public int method_9603(class_2680 blockState, class_1922 blockAccess, class_2338 pos, class_2350 side) {
        if (side != blockState.method_11654(field_10927))
            return 0;
        return method_9524(blockState, blockAccess, pos, side);
    }

    @Override
    public int method_9524(class_2680 state, class_1922 blockAccess, class_2338 pos, class_2350 side) {
        if (!state.method_11654(RECEIVER))
            return 0;
        return getBlockEntityOptional(blockAccess, pos).map(RedstoneLinkBlockEntity::getReceivedSignal).orElse(0);
    }

    @Override
    protected void method_9515(class_2689.class_2690<class_2248, class_2680> builder) {
        builder.method_11667(POWERED, RECEIVER);
        super.method_9515(builder);
    }

    @Override
    protected class_1269 method_55766(class_2680 state, class_1937 level, class_2338 pos, class_1657 player, class_3965 hitResult) {
        if (player.method_5715() && toggleMode(state, level, pos) == class_1269.field_5812) {
            level.method_64310(pos, this, 1);
            return class_1269.field_5812;
        }
        return class_1269.field_5811;
    }

    public class_1269 toggleMode(class_2680 state, class_1937 level, class_2338 pos) {
        if (level.method_8608())
            return class_1269.field_5812;

        return onBlockEntityUse(
            level, pos, be -> {
                Boolean wasReceiver = state.method_11654(RECEIVER);
                boolean blockPowered = level.method_49803(pos);
                level.method_8652(pos, state.method_28493(RECEIVER).method_11657(POWERED, blockPowered), class_2248.field_31036);
                be.transmit(wasReceiver ? 0 : getPower(level, state, pos));
                return class_1269.field_5812;
            }
        );
    }

    @Override
    public class_1269 onWrenched(class_2680 state, class_1838 context) {
        if (toggleMode(state, context.method_8045(), context.method_8037()) == class_1269.field_5812) {
            context.method_8045().method_64310(context.method_8037(), this, 1);
            return class_1269.field_5812;
        }
        return super.onWrenched(state, context);
    }

    @Override
    public class_2680 getRotatedBlockState(class_2680 originalState, class_2350 _targetedFace) {
        return originalState;
    }

    @Override
    public boolean canConnectRedstone(class_2680 state, class_2350 side) {
        return side != null;
    }

    @Override
    public boolean method_9558(class_2680 state, class_4538 worldIn, class_2338 pos) {
        class_2338 neighbourPos = pos.method_10093(state.method_11654(field_10927).method_10153());
        class_2680 neighbour = worldIn.method_8320(neighbourPos);
        return !neighbour.method_45474();
    }

    @Override
    public class_2680 method_9605(class_1750 context) {
        class_2680 state = method_9564();
        state = state.method_11657(field_10927, context.method_8038());
        return state;
    }

    @Override
    public class_265 method_9530(class_2680 state, class_1922 worldIn, class_2338 pos, class_3726 context) {
        return AllShapes.REDSTONE_BRIDGE.get(state.method_11654(field_10927));
    }

    @Override
    protected boolean method_9516(class_2680 state, class_10 pathComputationType) {
        return false;
    }

    @Override
    public Class<RedstoneLinkBlockEntity> getBlockEntityClass() {
        return RedstoneLinkBlockEntity.class;
    }

    @Override
    public class_2591<? extends RedstoneLinkBlockEntity> getBlockEntityType() {
        return AllBlockEntityTypes.REDSTONE_LINK;
    }

}

==> create-fly-6.0.7-1.21.8-19-sources.jar: com/zurrtum/create/content/redstone/link/RedstoneLinkBlock.java <==
package com.zurrtum.create.content.redstone.link;

import com.zurrtum.create.AllBlockEntityTypes;
import com.zurrtum.create.AllShapes;
import com.zurrtum.create.catnip.data.Iterate;
import com.zurrtum.create.foundation.block.IBE;
import com.zurrtum.create.foundation.block.NeighborUpdateListeningBlock;
import com.zurrtum.create.foundation.block.RedStoneConnectBlock;
import com.zurrtum.create.foundation.block.WrenchableDirectionalBlock;
import net.minecraft.class_10;
import net.minecraft.class_1269;
import net.minecraft.class_1657;
import net.minecraft.class_1750;
import net.minecraft.class_1838;
import net.minecraft.class_1922;
import net.minecraft.class_1937;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2591;
import net.minecraft.class_265;
import net.minecraft.class_2680;
import net.minecraft.class_2689;
import net.minecraft.class_2741;
import net.minecraft.class_2746;
import net.minecraft.class_3218;
import net.minecraft.class_3726;
import net.minecraft.class_3965;
import net.minecraft.class_4538;
import net.minecraft.class_5819;

public class RedstoneLinkBlock extends WrenchableDirectionalBlock implements IBE<RedstoneLinkBlockEntity>, RedStoneConnectBlock, NeighborUpdateListeningBlock {

    public static final class_2746 POWERED = class_2741.field_12484;
    public static final class_2746 RECEIVER = class_2746.method_11825("receiver");

    public RedstoneLinkBlock(class_2251 properties) {
        super(properties);
        method_9590(method_9564().method_11657(POWERED, false).method_11657(RECEIVER, false));
    }

    @Override
    public void neighborUpdate(class_2680 state, class_1937 level, class_2338 pos, class_2248 sourceBlock, class_2338 fromPos, boolean isMoving) {
        if (level.field_9236)
            return;
        if (fromPos.equals(pos.method_10093(state.method_11654(field_10927).method_10153()))) {
            if (!method_9558(state, level, pos)) {
                level.method_22352(pos, true);
                return;
            }
        }
        if (!level.method_8397().method_8677(pos, this))
            level.method_64310(pos, this, 1);
    }

    @Override
    public void method_9588(class_2680 state, class_3218 level, class_2338 pos, class_5819 r) {
        updateTransmittedSignal(state, level, pos);

        if (state.method_11654(RECEIVER))
            return;
        class_2350 attachedFace = state.method_11654(RedstoneLinkBlock.field_10927).method_10153();
        class_2338 attachedPos = pos.method_10093(attachedFace);
        level.method_8408(pos, level.method_8320(pos).method_26204());
        level.method_8408(attachedPos, level.method_8320(attachedPos).method_26204());
    }

    @Override
    public void method_9615(class_2680 state, class_1937 worldIn, class_2338 pos, class_2680 oldState, boolean isMoving) {
        if (state.method_26204() == oldState.method_26204() || isMoving)
            return;
        updateTransmittedSignal(state, worldIn, pos);
    }

    public void updateTransmittedSignal(class_2680 state, class_1937 worldIn, class_2338 pos) {
        if (worldIn.field_9236)
            return;
        if (state.method_11654(RECEIVER))
            return;

        int power = getPower(worldIn, pos);
        int powerFromPanels = getBlockEntityOptional(worldIn, pos).map(be -> {
            if (be.panelSupport == null)
                return 0;
            Boolean tri = be.panelSupport.shouldBePoweredTristate();
            if (tri == null)
                return -1;
            return tri ? 15 : 0;
        }).orElse(0);

        // Suppress update if an input panel exists but is not loaded
        if (powerFromPanels == -1)
            return;

        power = Math.max(power, powerFromPanels);

        boolean previouslyPowered = state.method_11654(POWERED);
        if (previouslyPowered != power > 0)
            worldIn.method_8652(pos, state.method_28493(POWERED), class_2248.field_31028);

        int transmit = power;
        withBlockEntityDo(worldIn, pos, be -> be.transmit(transmit));
    }

    private int getPower(class_1937 worldIn, class_2338 pos) {
        int power = 0;
        for (class_2350 direction : Iterate.directions)
            power = Math.max(worldIn.method_49808(pos.method_10093(direction), direction), power);
        for (class_2350 direction : Iterate.directions)
            power = Math.max(worldIn.method_49808(pos.method_10093(direction), class_2350.field_11036), power);
        return power;
    }

    @Override
    public boolean method_9506(class_2680 state) {
        return state.method_11654(POWERED) && state.method_11654(RECEIVER);
    }

    @Override
    public int method_9603(class_2680 blockState, class_1922 blockAccess, class_2338 pos, class_2350 side) {
        if (side != blockState.method_11654(field_10927))
            return 0;
        return method_9524(blockState, blockAccess, pos, side);
    }

    @Override
    public int method_9524(class_2680 state, class_1922 blockAccess, class_2338 pos, class_2350 side) {
        if (!state.method_11654(RECEIVER))
            return 0;
        return getBlockEntityOptional(blockAccess, pos).map(RedstoneLinkBlockEntity::getReceivedSignal).orElse(0);
    }

    @Override
    protected void method_9515(class_2689.class_2690<class_2248, class_2680> builder) {
        builder.method_11667(POWERED, RECEIVER);
        super.method_9515(builder);
    }

    @Override
    protected class_1269 method_55766(class_2680 state, class_1937 level, class_2338 pos, class_1657 player, class_3965 hitResult) {
        if (player.method_5715() && toggleMode(state, level, pos) == class_1269.field_5812) {
            level.method_64310(pos, this, 1);
            return class_1269.field_5812;
        }
        return class_1269.field_5811;
    }

    public class_1269 toggleMode(class_2680 state, class_1937 worldIn, class_2338 pos) {
        if (worldIn.field_9236)
            return class_1269.field_5812;

        return onBlockEntityUse(
            worldIn, pos, be -> {
                Boolean wasReceiver = state.method_11654(RECEIVER);
                boolean blockPowered = worldIn.method_49803(pos);
                worldIn.method_8652(pos, state.method_28493(RECEIVER).method_11657(POWERED, blockPowered), class_2248.field_31036);
                be.transmit(wasReceiver ? 0 : getPower(worldIn, pos));
                return class_1269.field_5812;
            }
        );
    }

    @Override
    public class_1269 onWrenched(class_2680 state, class_1838 context) {
        if (toggleMode(state, context.method_8045(), context.method_8037()) == class_1269.field_5812) {
            context.method_8045().method_64310(context.method_8037(), this, 1);
            return class_1269.field_5812;
        }
        return super.onWrenched(state, context);
    }

    @Override
    public class_2680 getRotatedBlockState(class_2680 originalState, class_2350 _targetedFace) {
        return originalState;
    }

    @Override
    public boolean canConnectRedstone(class_2680 state, class_2350 side) {
        return side != null;
    }

    @Override
    public boolean method_9558(class_2680 state, class_4538 worldIn, class_2338 pos) {
        class_2338 neighbourPos = pos.method_10093(state.method_11654(field_10927).method_10153());
        class_2680 neighbour = worldIn.method_8320(neighbourPos);
        return !neighbour.method_45474();
    }

    @Override
    public class_2680 method_9605(class_1750 context) {
        class_2680 state = method_9564();
        state = state.method_11657(field_10927, context.method_8038());
        return state;
    }

    @Override
    public class_265 method_9530(class_2680 state, class_1922 worldIn, class_2338 pos, class_3726 context) {
        return AllShapes.REDSTONE_BRIDGE.get(state.method_11654(field_10927));
    }

    @Override
    protected boolean method_9516(class_2680 state, class_10 pathComputationType) {
        return false;
    }

    @Override
    public Class<RedstoneLinkBlockEntity> getBlockEntityClass() {
        return RedstoneLinkBlockEntity.class;
    }

    @Override
    public class_2591<? extends RedstoneLinkBlockEntity> getBlockEntityType() {
        return AllBlockEntityTypes.REDSTONE_LINK;
    }

}
